// maincr1.cpp
// requires random.cpp

#include "random.h"
#include <iostream>
#include <cmath>
#include <algorithm>

using namespace std;
int main()
{
    cout << "\n *** START CR1: Monte Carlo Merton Model *** \n";

    // STEP 1: INPUT PARAMETERS

   	double T=4; // maturity
    double D=70; // debt of the firm
   	double V0=100; // initial assets of the firm
   	double sigma=0.20; // volatility of the assets of the firm
   	double r=0.05; // risk free interest rate
   	int N=500; // number of steps
   	int M=10000; // number of simulations
   	double V[N+1];
    double sumpayoff=0;
    double premium=0;
    double dt = T / N;
    double default_count = 0.0;

    // STEP 2: MAIN SIMULATION LOOP

    for (int j=0; j < M; j++)
   	{
        V[0]=V0;

        // STEP 3: TIME INTEGRATION LOOP

        for (int i=0; i < N; i++)
        {
            double epsilon = SampleBoxMuller();
            V[i+1] = V[i]*(1+r*dt+sigma*sqrt(dt)*epsilon);
        }

        // STEP 4: COMPUTE PAYOFF
        sumpayoff += max(V[N]-D,0.0);

        // STEP 5: COMPUTE NUMBER DEFAULTS
        if (V[N]>D)
        {
           default_count=default_count+1;
        }

    }

    // STEP 6: COMPUTE DISCOUNTED EXPECTED PAYOFF
    // EQUITY VALUE (E) AT TIME=0

   	premium =  exp(-r*T)*(sumpayoff / M);

    // STEP 7: OUTPUT RESULTS

    cout <<"equity payoff (E(0)) =  " << premium  << "\n";
    cout <<"percentage defaults =  " << default_count/M*100 << " percent \n";
    cout << "\n *** END CR1: Monte Carlo Merton Model *** \n";

	return 0;
}
