// mainfx1.cpp

#include <iostream>
#include <cmath>

using namespace std;
int main()
{
    cout << "\n *** START FX1: Finite Difference European Call *** \n";

    // STEP 1: INPUT PARAMETERS
   	double T=0.5; // maturity
    double K=75; // strike
   	double S0=75; // spot
   	double sigma=0.30; // volatility
   	double r=0.05; // interest rate

   	int N=5; // number of space steps
   	int M=6; // number of time steps
    double dt = 0.1; // time step
   	double dx = 0.5; // space step

   	double u[10][10];
   	double v[10][10];

    double dtau = dt * (0.5*sigma*sigma);
    double alpha = dtau/(dx*dx);
    double k = r/(0.5*sigma*sigma);

    cout <<" dtau = "<< dtau << "\n";
    cout <<" alpha = "<< alpha << "\n";
    cout <<" k = "<< k << "\n";

   	double xmin=-1;
   	double xmax=+1;
   	double x[10];
   	double tau[10];
    double S[10];
   	double t[10];
    double premium=0;

cout <<".....x and tau grids ....................... "<< "\n";

    // STEP 2: SETUP MESH
    for (int i=0; i < N; i++)
    {
        x[i] = xmin + i*dx;
        S[i]= K*exp(x[i]);
        cout <<"x " << x[i] << "\n";
        cout <<"S " << S[i] << "\n";
    }
    for (int j=0; j < M; j++)
    {
        t[j] = j*dt;
        tau[j]=(T-t[j])/(0.5*sigma*sigma);
        cout <<"t " << t[j] << "\n";
        cout <<"tau " << tau[j] << "\n";
    }

     cout <<".....init cond........................ "<< "\n";

    // STEP 3: SETUP INITIAL CONDITION
    for (int i=0; i < N; i++)
    {
        u[i][0] = max(exp(0.5*(k+1)*x[i])-exp(0.5*(k-1)*x[i]),0.0);
        cout <<"i " << i << "u " << u[i][0] << "\n";
    }

cout <<".....bcs........................ "<< "\n";

    // STEP 4: SETUP BOUNDARY CONDITIONS
    for (int j=1; j < M; j++)
    {
        u[0][j] = 0.0;
        u[N-1][j] = u[N-1][0];
        cout <<"j " << j << " u[0][j] " << u[0][j] << "\n";
        cout <<"j " << j << " u[N-1][j] " << u[N-1][j] << "\n";
    }

    // STEP 5: COMPUTE FORWARD DIFFERENCES
    for (int j=0; j < M-1; j++)
    {
        for (int i=1; i < N-1; i++)
        {
        u[i][j+1] = alpha*u[i+1][j]+(1-2*alpha)*u[i][j]+alpha*u[i-1][j];
        }
    }

cout <<".....u and v....................... "<< "\n";

    // STEP 6: TRANSFORM SOLUTION FROM X TO S COORDINATES
    for (int j=0; j < M; j++)
    {
        for (int i=0; i < N; i++)
        {
        v[i][j] = pow(K,(0.5*(1+k)))*pow(S[i],(0.5*(1-k)))*exp(1.0/8.0*(k+1)*(k+1)*sigma*sigma*(T-t[i]))*u[i][j];
        cout <<"i= " << i <<", j= " << j << " v[i][j] " << v[i][j] << "\n";
        }
    }

    cout <<"premium =  " << v[2][5] << "\n"; // for strike = 75
    cout << "\n *** END FX1: Finite Difference European Call *** \n";

	return 0;
}
